﻿using System;
using System.Collections.Generic;
using System.Text;

namespace LASRead.LASFormat
{
    
    class GeoKeyTag // Tag = 34735
    {
        // Header, first 8 bytes
        short KeyDirectoryVersion;
        short KeyRevision;
        short MinorRevision;
        short NumberOfKeys;



        /// <summary>
        /// GeoTIFF Tags that are included to contain the format (especially coordinate) data of the file
        /// Due to the structure of a TIFF file, data are stored in Keys - these keys often further link to data in another tag
        /// LiDAR capture data from a section of New Zealand contianed the following keys:
        ///     1       1       0       11      -- ?
        ///     1024    0       1       1       -- Type of coordinates (1 == Projected System)
        ///     1025    0       1       1       -- Raster system used  (1 == Pixel is area)
        ///     1026    34737   66      0       -- ASCII Text about the file (Specifically, 'NZGD2000 / New Zealand Transverse Mercator 2000 + NZVD2016 height' - between 0-66 bytes of tag 34737)
        ///     2049    34737   9       66      -- ASCII Geographic Citation Parameters, between 66-66+9 bytes ('NZGD2000')
        ///     2054    0       1       9102    -- Angular Units (9102 = Degrees)
        ///     3072    0       1       2193    -- Projected Coordinate System Type (2193 is not an expected value
        ///     3076    0       1       9001    -- Projection Units (9001 = meters)
        ///     4096    0       1       7839    -- Vertical Coordinate System (7839 = NZVD2016)
        ///     4097    34737   16      75      -- ASCII Vertical System Name ('NZVD2016 height')
        ///     4098    0       1       1169    -- Vertical Datum
        ///     4099    0       1       9001    -- Vertical Units (meters)
        /// </summary>
        struct Key
        {
            public short KeyID;
            public short TIFFTagLocation;
            public short Count;
            public short Value_Offset;
        }



        /// <summary>
        /// Linear unit specifications - used in key ID 3076
        /// </summary>
        enum Units
        {
            Undefined = 0,
            Linear_Meter = 9001,
            Linear_Foot = 9002,
            Linear_Foot_US_Survey = 9003,
            Linear_Foot_Modified_American = 9004,
            Linear_Foot_Clarke = 9005,
            Linear_Foot_Indian = 9006,
            Linear_Link = 9007,
            Linear_Link_Benoit = 9008,
            Linear_Link_Sears = 9009,
            Linear_Chain_Benoit = 9010,
            Linear_Chain_Sears = 9011,
            Linear_Yard_Sears = 9012,
            Linear_Yard_Indian = 9013,
            Linear_Fathom = 9014,
            Linear_Mile_International_Nautical = 9015,

            Angular_Radian = 9101,
            Angular_Degree = 9102,
            Angular_Arc_Minute = 9103,
            Angular_Arc_Second = 9104,
            Angular_Grad = 9105,
            Angular_Gon = 9106,
            Angular_DMS = 9107,
            Angular_DMS_Hemisphere = 9108,
        }

        /// <summary>
        /// Vertical system codes - (key 4096)
        /// </summary>
        enum VerticalTypeCodes
        {
            VertCS_Airy_1830_ellipsoid = 5001,
            VertCS_Airy_Modified_1849_ellipsoid = 5002,
            VertCS_ANS_ellipsoid = 5003,
            VertCS_Bessel_1841_ellipsoid = 5004,
            VertCS_Bessel_Modified_ellipsoid = 5005,
            VertCS_Bessel_Namibia_ellipsoid = 5006,
            VertCS_Clarke_1858_ellipsoid = 5007,
            VertCS_Clarke_1866_ellipsoid = 5008,
            VertCS_Clarke_1880_Benoit_ellipsoid = 5010,
            VertCS_Clarke_1880_IGN_ellipsoid = 5011,
            VertCS_Clarke_1880_RGS_ellipsoid = 5012,
            VertCS_Clarke_1880_Arc_ellipsoid = 5013,
            VertCS_Clarke_1880_SGA_1922_ellipsoid = 5014,
            VertCS_Everest_1830_1937_Adjustment_ellipsoid = 5015,
            VertCS_Everest_1830_1967_Definition_ellipsoid = 5016,
            VertCS_Everest_1830_1975_Definition_ellipsoid = 5017,
            VertCS_Everest_1830_Modified_ellipsoid = 5018,
            VertCS_GRS_1980_ellipsoid = 5019,
            VertCS_Helmert_1906_ellipsoid = 5020,
            VertCS_INS_ellipsoid = 5021,
            VertCS_International_1924_ellipsoid = 5022,
            VertCS_International_1967_ellipsoid = 5023,
            VertCS_Krassowsky_1940_ellipsoid = 5024,
            VertCS_NWL_9D_ellipsoid = 5025,
            VertCS_NWL_10D_ellipsoid = 5026,
            VertCS_Plessis_1817_ellipsoid = 5027,
            VertCS_Struve_1860_ellipsoid = 5028,
            VertCS_War_Office_ellipsoid = 5029,
            VertCS_WGS_84_ellipsoid = 5030,
            VertCS_GEM_10C_ellipsoid = 5031,
            VertCS_OSU86F_ellipsoid = 5032,
            VertCS_OSU91A_ellipsoid = 5033,
            VertCS_Newlyn = 5101,
            VertCS_North_American_Vertical_Datum_1929 = 5102,
            VertCS_North_American_Vertical_Datum_1988 = 5103,
            VertCS_Yellow_Sea_1956 = 5104,
            VertCS_Baltic_Sea = 5105,
            VertCS_Caspian_Sea = 5106,

            NZ_Vertical = 7839
        }
    }
}
